@@ -215,7 +215,6 @@ delete_cygwin_fd (assuan_fd_t sockfd)
 }
 
 
-#ifdef HAVE_W32CE_SYSTEM
 static wchar_t *
 utf8_to_wchar (const char *string)
 {
@@ -289,10 +288,7 @@ MyDeleteFile (LPCSTR lpFileName)
   SetLastError (err);
   return result;
 }
-#else /*!HAVE_W32CE_SYSTEM*/
-#define MyCreateFile CreateFileA
-#define MyDeleteFile DeleteFileA
-#endif /*!HAVE_W32CE_SYSTEM*/
+
 
 int
 _assuan_sock_wsa2errno (int err)
@@ -345,17 +341,17 @@ static int
 read_port_and_nonce (const char *fname, unsigned short *port, char *nonce,
                      int *cygwin)
 {
-  FILE *fp;
+  estream_t fp;
   char buffer[50], *p;
   size_t nread;
   int aval;
 
   *cygwin = 0;
-  fp = fopen (fname, "rb");
+  fp = gpgrt_fopen (fname, "rb");
   if (!fp)
     return -1;
-  nread = fread (buffer, 1, sizeof buffer - 1, fp);
-  fclose (fp);
+  nread = gpgrt_fread (buffer, 1, sizeof buffer - 1, fp);
+  gpgrt_fclose (fp);
   if (!nread)
     {
       gpg_err_set_errno (ENOENT);
@@ -690,6 +686,8 @@ do_writen (assuan_context_t ctx, assuan_fd_t sockfd,
 }
 
 
+#define TIMEOUT_NOT_WAITING_SOCKS5_FOREVER 1 /* in second(s) */
+
 /* Connect using the SOCKS5 protocol. */
 static int
 socks5_connect (assuan_context_t ctx, assuan_fd_t sock,
@@ -703,13 +701,23 @@ socks5_connect (assuan_context_t ctx, assuan_fd_t sock,
   struct sockaddr_in  proxyaddr_in;
   struct sockaddr *proxyaddr;
   size_t proxyaddrlen;
-  struct sockaddr_in6 *addr_in6;
-  struct sockaddr_in  *addr_in;
+  union {
+    struct sockaddr *addr;
+    struct sockaddr_in *addr_in;
+    struct sockaddr_in6 *addr_in6;
+  } addru;
   unsigned char buffer[22+512]; /* The extra 512 gives enough space
                                    for username/password or the
                                    hostname. */
   size_t buflen, hostnamelen;
   int method;
+  fd_set fds;
+  struct timeval tv = { TIMEOUT_NOT_WAITING_SOCKS5_FOREVER, 0 };
+
+  addru.addr = addr;
+
+  FD_ZERO (&fds);
+  FD_SET (HANDLE2SOCKET (sock), &fds);
 
   /* memset (&proxyaddr_in6, 0, sizeof proxyaddr_in6); */
   memset (&proxyaddr_in, 0, sizeof proxyaddr_in);
@@ -765,6 +773,25 @@ socks5_connect (assuan_context_t ctx, assuan_fd_t sock,
   ret = do_writen (ctx, sock, buffer, 3);
   if (ret)
     return ret;
+
+  /* There may be a different service at the port, which doesn't
+     respond.  Not to be bothred by such a service.  */
+  /* FIXME: Since the process may block on select, it should be
+     npth_select to release thread scheduling if nPth is enabled.
+     Ideally, select is better to be in the system hooks.  However, it
+     is considered OK to use select directly; Normal use case is three
+     steps: detect SOCKS5 service before nPth use, configure nPth
+     system hooks, and then use socks5_connect.  For the first call,
+     select indeed blocks, but it's only single thread.  For
+     succeeding calls, this select should soon return successfully.
+   */
+  ret = select (HANDLE2SOCKET (sock)+1, &fds, NULL, NULL, &tv);
+  if (!ret)
+    {
+      gpg_err_set_errno (ETIMEDOUT);
+      return -1;
+    }
+
   ret = do_readn (ctx, sock, buffer, 2);
   if (ret)
     return ret;
@@ -848,20 +875,16 @@ socks5_connect (assuan_context_t ctx, assuan_fd_t sock,
     }
   else if (addr->sa_family == AF_INET6)
     {
-      addr_in6 = (struct sockaddr_in6 *)addr;
-
       buffer[3] = 4; /* ATYP = IPv6 */
-      memcpy (buffer+ 4, &addr_in6->sin6_addr.s6_addr, 16); /* DST.ADDR */
-      memcpy (buffer+20, &addr_in6->sin6_port, 2);          /* DST.PORT */
+      memcpy (buffer+ 4, &addru.addr_in6->sin6_addr.s6_addr, 16); /* DST.ADDR */
+      memcpy (buffer+20, &addru.addr_in6->sin6_port, 2);          /* DST.PORT */
       buflen = 22;
     }
   else
     {
-      addr_in = (struct sockaddr_in *)addr;
-
       buffer[3] = 1; /* ATYP = IPv4 */
-      memcpy (buffer+4, &addr_in->sin_addr.s_addr, 4); /* DST.ADDR */
-      memcpy (buffer+8, &addr_in->sin_port, 2);        /* DST.PORT */
+      memcpy (buffer+4, &addru.addr_in->sin_addr.s_addr, 4); /* DST.ADDR */
+      memcpy (buffer+8, &addru.addr_in->sin_port, 2);        /* DST.PORT */
       buflen = 10;
     }
   ret = do_writen (ctx, sock, buffer, buflen);
@@ -931,15 +954,22 @@ socks5_connect (assuan_context_t ctx, assuan_fd_t sock,
 static int
 use_socks (struct sockaddr *addr)
 {
+  union {
+    struct sockaddr *addr;
+    struct sockaddr_in *addr_in;
+    struct sockaddr_in6 *addr_in6;
+  } addru;
+
+  addru.addr = addr;
+
   if (!tor_mode)
     return 0;
   else if (addr->sa_family == AF_INET6)
     {
-      struct sockaddr_in6 *addr_in6 = (struct sockaddr_in6 *)addr;
       const unsigned char *s;
       int i;
 
-      s = (unsigned char *)&addr_in6->sin6_addr.s6_addr;
+      s = (unsigned char *)&addru.addr_in6->sin6_addr.s6_addr;
       if (s[15] != 1)
         return 1;   /* Last octet is not 1 - not the loopback address.  */
       for (i=0; i < 15; i++, s++)
@@ -950,9 +980,7 @@ use_socks (struct sockaddr *addr)
     }
   else if (addr->sa_family == AF_INET)
     {
-      struct sockaddr_in *addr_in = (struct sockaddr_in *)addr;
-
-      if (*(unsigned char*)&addr_in->sin_addr.s_addr == 127)
+      if (*(unsigned char*)&addru.addr_in->sin_addr.s_addr == 127)
         return 0; /* Loopback (127.0.0.0/8) */
 
       return 1;
@@ -1495,3 +1523,10 @@ assuan_sock_check_nonce (assuan_fd_t fd, assuan_sock_nonce_t *nonce)
 {
   return _assuan_sock_check_nonce (sock_ctx, fd, nonce);
 }
+
+void
+assuan_sock_set_system_hooks (assuan_system_hooks_t system_hooks)
+{
+  if (sock_ctx)
+    _assuan_system_hooks_copy (&sock_ctx->system, system_hooks);
+}
