@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -29,6 +29,7 @@
 #include "keybox-defs.h"
 #include "../common/sysutils.h"
 #include "../common/host2net.h"
+#include "../common/utilproto.h"
 
 #define EXTSEP_S "."
 
@@ -68,88 +69,27 @@ fseeko (FILE * stream, off_t newpos, int whence)
 #endif /* !defined(HAVE_FSEEKO) && !defined(fseeko) */
 
 
-
 static int
 create_tmp_file (const char *template,
-                 char **r_bakfname, char **r_tmpfname, FILE **r_fp)
+                 char **r_bakfname, char **r_tmpfname, estream_t *r_fp)
 {
-  char *bakfname, *tmpfname;
+  gpg_error_t err;
 
-  *r_bakfname = NULL;
-  *r_tmpfname = NULL;
-
-# ifdef USE_ONLY_8DOT3
-  /* Here is another Windoze bug?:
-   * you can't rename("pubring.kbx.tmp", "pubring.kbx");
-   * but	rename("pubring.kbx.tmp", "pubring.aaa");
-   * works.  So we replace ".kbx" by ".kb_" or ".k__".  Note that we
-   * can't use ".bak" and ".tmp", because these suffixes are used by
-   * gpg and would lead to a sharing violation or data corruption.
-   */
-  if (strlen (template) > 4
-      && !strcmp (template+strlen(template)-4, EXTSEP_S "kbx") )
+  err = keybox_tmp_names (template, 0, r_bakfname, r_tmpfname);
+  if (!err)
     {
-      bakfname = xtrymalloc (strlen (template) + 1);
-      if (!bakfname)
-        return gpg_error_from_syserror ();
-      strcpy (bakfname, template);
-      strcpy (bakfname+strlen(template)-4, EXTSEP_S "kb_");
-
-      tmpfname = xtrymalloc (strlen (template) + 1);
-      if (!tmpfname)
+      *r_fp = es_fopen (*r_tmpfname, "wb");
+      if (!*r_fp)
         {
-          gpg_error_t tmperr = gpg_error_from_syserror ();
-          xfree (bakfname);
-          return tmperr;
+          err = gpg_error_from_syserror ();
+          xfree (*r_tmpfname);
+          *r_tmpfname = NULL;
+          xfree (*r_bakfname);
+          *r_bakfname = NULL;
         }
-      strcpy (tmpfname,template);
-      strcpy (tmpfname + strlen (template)-4, EXTSEP_S "k__");
-    }
-  else
-    { /* File does not end with kbx, thus we hope we are working on a
-         modern file system and appending a suffix works. */
-      bakfname = xtrymalloc ( strlen (template) + 5);
-      if (!bakfname)
-        return gpg_error_from_syserror ();
-      strcpy (stpcpy (bakfname, template), EXTSEP_S "kb_");
-
-      tmpfname = xtrymalloc ( strlen (template) + 5);
-      if (!tmpfname)
-        {
-          gpg_error_t tmperr = gpg_error_from_syserror ();
-          xfree (bakfname);
-          return tmperr;
-        }
-      strcpy (stpcpy (tmpfname, template), EXTSEP_S "k__");
-    }
-# else /* Posix file names */
-  bakfname = xtrymalloc (strlen (template) + 2);
-  if (!bakfname)
-    return gpg_error_from_syserror ();
-  strcpy (stpcpy (bakfname,template),"~");
-
-  tmpfname = xtrymalloc ( strlen (template) + 5);
-  if (!tmpfname)
-    {
-      gpg_error_t tmperr = gpg_error_from_syserror ();
-      xfree (bakfname);
-      return tmperr;
-    }
-  strcpy (stpcpy (tmpfname,template), EXTSEP_S "tmp");
-# endif /* Posix filename */
-
-  *r_fp = fopen (tmpfname, "wb");
-  if (!*r_fp)
-    {
-      gpg_error_t tmperr = gpg_error_from_syserror ();
-      xfree (tmpfname);
-      xfree (bakfname);
-      return tmperr;
     }
 
-  *r_bakfname = bakfname;
-  *r_tmpfname = tmpfname;
-  return 0;
+  return err;
 }
 
 
@@ -158,6 +98,7 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
                  const char *fname, int secret )
 {
   int rc=0;
+  int block = 0;
 
   /* restrict the permissions for secret keyboxs */
 #ifndef HAVE_DOSISH_SYSTEM
@@ -180,34 +121,35 @@ rename_tmp_file (const char *bakfname, const char *tmpfname,
   /* First make a backup file except for secret keyboxes. */
   if (!secret)
     {
-#if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
-      gnupg_remove (bakfname);
-#endif
-      if (rename (fname, bakfname) )
-        {
-          return gpg_error_from_syserror ();
-	}
+      block = 1;
+      rc = gnupg_rename_file (fname, bakfname, &block);
+      if (rc)
+        goto leave;
     }
 
   /* Then rename the file. */
-#if defined(HAVE_DOSISH_SYSTEM) || defined(__riscos__)
-  gnupg_remove (fname);
-#endif
-  if (rename (tmpfname, fname) )
+  rc = gnupg_rename_file (tmpfname, fname, NULL);
+  if (block)
     {
-      rc = gpg_error_from_syserror ();
-      if (secret)
-        {
-/*            log_info ("WARNING: 2 files with confidential" */
-/*                       " information exists.\n"); */
-/*            log_info ("%s is the unchanged one\n", fname ); */
-/*            log_info ("%s is the new one\n", tmpfname ); */
-/*            log_info ("Please fix this possible security flaw\n"); */
-	}
-      return rc;
+      gnupg_unblock_all_signals ();
+      block = 0;
     }
+  /* if (rc) */
+  /*   { */
+  /*     if (secret) */
+  /*       { */
+  /*         log_info ("WARNING: 2 files with confidential" */
+  /*                   " information exists.\n"); */
+  /*         log_info ("%s is the unchanged one\n", fname ); */
+  /*         log_info ("%s is the new one\n", tmpfname ); */
+  /*         log_info ("Please fix this possible security flaw\n"); */
+  /*       } */
+  /*   } */
 
-  return 0;
+ leave:
+  if (block)
+    gnupg_unblock_all_signals ();
+  return rc;
 }
 
 
@@ -219,8 +161,9 @@ static int
 blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
                int secret, int for_openpgp, off_t start_offset)
 {
-  FILE *fp, *newfp;
-  int rc=0;
+  gpg_err_code_t ec;
+  estream_t fp, newfp;
+  int rc = 0;
   char *bakfname = NULL;
   char *tmpfname = NULL;
   char buffer[4096];  /* (Must be at least 32 bytes) */
@@ -228,33 +171,33 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
 
   /* Open the source file. Because we do a rename, we have to check the
      permissions of the file */
-  if (access (fname, W_OK))
-    return gpg_error_from_syserror ();
+  if ((ec = gnupg_access (fname, W_OK)))
+    return gpg_error (ec);
 
-  fp = fopen (fname, "rb");
+  fp = es_fopen (fname, "rb");
   if (mode == FILECOPY_INSERT && !fp && errno == ENOENT)
     {
       /* Insert mode but file does not exist:
          Create a new keybox file. */
-      newfp = fopen (fname, "wb");
+      newfp = es_fopen (fname, "wb");
       if (!newfp )
         return gpg_error_from_syserror ();
 
       rc = _keybox_write_header_blob (newfp, for_openpgp);
       if (rc)
         {
-          fclose (newfp);
+          es_fclose (newfp);
           return rc;
         }
 
-      rc = _keybox_write_blob (blob, newfp);
+      rc = _keybox_write_blob (blob, newfp, NULL);
       if (rc)
         {
-          fclose (newfp);
+          es_fclose (newfp);
           return rc;
         }
 
-      if ( fclose (newfp) )
+      if ( es_fclose (newfp) )
         return gpg_error_from_syserror ();
 
 /*        if (chmod( fname, S_IRUSR | S_IWUSR )) */
@@ -271,12 +214,11 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
       goto leave;
     }
 
-  /* Create the new file. */
+  /* Create the new file.  On success NEWFP is initialized.  */
   rc = create_tmp_file (fname, &bakfname, &tmpfname, &newfp);
   if (rc)
     {
-      fclose (fp);
-      fclose (newfp);
+      es_fclose (fp);
       goto leave;
     }
 
@@ -288,7 +230,7 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
       /* Copy everything to the new file.  If this is for OpenPGP, we
          make sure that the openpgp flag is set in the header.  (We
          failsafe the blob type.) */
-      while ( (nread = fread (buffer, 1, DIM(buffer), fp)) > 0 )
+      while ( (nread = es_fread (buffer, 1, DIM(buffer), fp)) > 0 )
         {
           if (first_record && for_openpgp
               && buffer[4] == KEYBOX_BLOBTYPE_HEADER)
@@ -297,19 +239,19 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
               buffer[7] |= 0x02; /* OpenPGP data may be available.  */
             }
 
-          if (fwrite (buffer, nread, 1, newfp) != 1)
+          if (es_fwrite (buffer, nread, 1, newfp) != 1)
             {
               rc = gpg_error_from_syserror ();
-              fclose (fp);
-              fclose (newfp);
+              es_fclose (fp);
+              es_fclose (newfp);
               goto leave;
             }
         }
-      if (ferror (fp))
+      if (es_ferror (fp))
         {
           rc = gpg_error_from_syserror ();
-          fclose (fp);
-          fclose (newfp);
+          es_fclose (fp);
+          es_fclose (newfp);
           goto leave;
         }
     }
@@ -325,33 +267,33 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
           nbytes = DIM(buffer);
           if (current + nbytes > start_offset)
               nbytes = start_offset - current;
-          nread = fread (buffer, 1, nbytes, fp);
+          nread = es_fread (buffer, 1, nbytes, fp);
           if (!nread)
             break;
           current += nread;
 
-          if (fwrite (buffer, nread, 1, newfp) != 1)
+          if (es_fwrite (buffer, nread, 1, newfp) != 1)
             {
               rc = gpg_error_from_syserror ();
-              fclose (fp);
-              fclose (newfp);
+              es_fclose (fp);
+              es_fclose (newfp);
               goto leave;
             }
         }
-      if (ferror (fp))
+      if (es_ferror (fp))
         {
           rc = gpg_error_from_syserror ();
-          fclose (fp);
-          fclose (newfp);
+          es_fclose (fp);
+          es_fclose (newfp);
           goto leave;
         }
 
       /* Skip this blob. */
-      rc = _keybox_read_blob (NULL, fp);
+      rc = _keybox_read_blob (NULL, fp, NULL);
       if (rc)
         {
-          fclose (fp);
-          fclose (newfp);
+          es_fclose (fp);
+          es_fclose (newfp);
           return rc;
         }
     }
@@ -359,11 +301,11 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
   /* Do an insert or update. */
   if ( mode == FILECOPY_INSERT || mode == FILECOPY_UPDATE )
     {
-      rc = _keybox_write_blob (blob, newfp);
+      rc = _keybox_write_blob (blob, newfp, NULL);
       if (rc)
         {
-          fclose (fp);
-          fclose (newfp);
+          es_fclose (fp);
+          es_fclose (newfp);
           return rc;
         }
     }
@@ -371,33 +313,33 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
   /* Copy the rest of the packet for an delete or update. */
   if (mode == FILECOPY_DELETE || mode == FILECOPY_UPDATE)
     {
-      while ( (nread = fread (buffer, 1, DIM(buffer), fp)) > 0 )
+      while ( (nread = es_fread (buffer, 1, DIM(buffer), fp)) > 0 )
         {
-          if (fwrite (buffer, nread, 1, newfp) != 1)
+          if (es_fwrite (buffer, nread, 1, newfp) != 1)
             {
               rc = gpg_error_from_syserror ();
-              fclose (fp);
-              fclose (newfp);
+              es_fclose (fp);
+              es_fclose (newfp);
               goto leave;
             }
         }
-      if (ferror (fp))
+      if (es_ferror (fp))
         {
           rc = gpg_error_from_syserror ();
-          fclose (fp);
-          fclose (newfp);
+          es_fclose (fp);
+          es_fclose (newfp);
           goto leave;
         }
     }
 
   /* Close both files. */
-  if (fclose(fp))
+  if (es_fclose(fp))
     {
       rc = gpg_error_from_syserror ();
-      fclose (newfp);
+      es_fclose (newfp);
       goto leave;
     }
-  if (fclose(newfp))
+  if (es_fclose(newfp))
     {
       rc = gpg_error_from_syserror ();
       goto leave;
@@ -412,12 +354,9 @@ blob_filecopy (int mode, const char *fname, KEYBOXBLOB blob,
 }
 
 
-/* Insert the OpenPGP keyblock {IMAGE,IMAGELEN} into HD.  SIGSTATUS is
-   a vector describing the status of the signatures; its first element
-   gives the number of following elements.  */
+/* Insert the OpenPGP keyblock {IMAGE,IMAGELEN} into HD. */
 gpg_error_t
-keybox_insert_keyblock (KEYBOX_HANDLE hd, const void *image, size_t imagelen,
-                        u32 *sigstatus)
+keybox_insert_keyblock (KEYBOX_HANDLE hd, const void *image, size_t imagelen)
 {
   gpg_error_t err;
   const char *fname;
@@ -444,7 +383,7 @@ keybox_insert_keyblock (KEYBOX_HANDLE hd, const void *image, size_t imagelen,
     return err;
   assert (nparsed <= imagelen);
   err = _keybox_create_openpgp_blob (&blob, &info, image, imagelen,
-                                     sigstatus, hd->ephemeral);
+                                      hd->ephemeral);
   _keybox_destroy_openpgp_info (&info);
   if (!err)
     {
@@ -485,7 +424,7 @@ keybox_update_keyblock (KEYBOX_HANDLE hd, const void *image, size_t imagelen)
   if (off == (off_t)-1)
     return gpg_error (GPG_ERR_GENERAL);
 
-  /* Close this the file so that we do no mess up the position for a
+  /* Close the file so that we do no mess up the position for a
      next search.  */
   _keybox_close_file (hd);
 
@@ -495,7 +434,7 @@ keybox_update_keyblock (KEYBOX_HANDLE hd, const void *image, size_t imagelen)
     return err;
   assert (nparsed <= imagelen);
   err = _keybox_create_openpgp_blob (&blob, &info, image, imagelen,
-                                     NULL, hd->ephemeral);
+                                     hd->ephemeral);
   _keybox_destroy_openpgp_info (&info);
 
   /* Update the keyblock.  */
@@ -565,7 +504,7 @@ keybox_set_flags (KEYBOX_HANDLE hd, int what, int idx, unsigned int value)
 {
   off_t off;
   const char *fname;
-  FILE *fp;
+  estream_t fp;
   gpg_err_code_t ec;
   size_t flag_pos, flag_size;
   const unsigned char *buffer;
@@ -597,12 +536,12 @@ keybox_set_flags (KEYBOX_HANDLE hd, int what, int idx, unsigned int value)
   off += flag_pos;
 
   _keybox_close_file (hd);
-  fp = fopen (hd->kb->fname, "r+b");
+  fp = es_fopen (hd->kb->fname, "r+b");
   if (!fp)
     return gpg_error_from_syserror ();
 
   ec = 0;
-  if (fseeko (fp, off, SEEK_SET))
+  if (es_fseeko (fp, off, SEEK_SET))
     ec = gpg_err_code_from_syserror ();
   else
     {
@@ -618,7 +557,7 @@ keybox_set_flags (KEYBOX_HANDLE hd, int what, int idx, unsigned int value)
         case 1:
         case 2:
         case 4:
-          if (fwrite (tmp+4-flag_size, flag_size, 1, fp) != 1)
+          if (es_fwrite (tmp+4-flag_size, flag_size, 1, fp) != 1)
             ec = gpg_err_code_from_syserror ();
           break;
         default:
@@ -627,7 +566,7 @@ keybox_set_flags (KEYBOX_HANDLE hd, int what, int idx, unsigned int value)
         }
     }
 
-  if (fclose (fp))
+  if (es_fclose (fp))
     {
       if (!ec)
         ec = gpg_err_code_from_syserror ();
@@ -643,7 +582,7 @@ keybox_delete (KEYBOX_HANDLE hd)
 {
   off_t off;
   const char *fname;
-  FILE *fp;
+  estream_t fp;
   int rc;
 
   if (!hd)
@@ -662,18 +601,18 @@ keybox_delete (KEYBOX_HANDLE hd)
   off += 4;
 
   _keybox_close_file (hd);
-  fp = fopen (hd->kb->fname, "r+b");
+  fp = es_fopen (hd->kb->fname, "r+b");
   if (!fp)
     return gpg_error_from_syserror ();
 
-  if (fseeko (fp, off, SEEK_SET))
+  if (es_fseeko (fp, off, SEEK_SET))
     rc = gpg_error_from_syserror ();
-  else if (putc (0, fp) == EOF)
+  else if (es_fputc (0, fp) == EOF)
     rc = gpg_error_from_syserror ();
   else
     rc = 0;
 
-  if (fclose (fp))
+  if (es_fclose (fp))
     {
       if (!rc)
         rc = gpg_error_from_syserror ();
@@ -688,9 +627,10 @@ keybox_delete (KEYBOX_HANDLE hd)
 int
 keybox_compress (KEYBOX_HANDLE hd)
 {
+  gpg_err_code_t ec;
   int read_rc, rc;
   const char *fname;
-  FILE *fp, *newfp;
+  estream_t fp, newfp;
   char *bakfname = NULL;
   char *tmpfname = NULL;
   int first_blob;
@@ -713,10 +653,10 @@ keybox_compress (KEYBOX_HANDLE hd)
 
   /* Open the source file. Because we do a rename, we have to check the
      permissions of the file */
-  if (access (fname, W_OK))
-    return gpg_error_from_syserror ();
+  if ((ec = gnupg_access (fname, W_OK)))
+    return gpg_error (ec);
 
-  fp = fopen (fname, "rb");
+  fp = es_fopen (fname, "rb");
   if (!fp && errno == ENOENT)
     return 0; /* Ready. File has been deleted right after the access above. */
   if (!fp)
@@ -727,7 +667,7 @@ keybox_compress (KEYBOX_HANDLE hd)
 
   /* A quick test to see if we need to compress the file at all.  We
      schedule a compress run after 3 hours. */
-  if ( !_keybox_read_blob (&blob, fp) )
+  if ( !_keybox_read_blob (&blob, fp, NULL) )
     {
       const unsigned char *buffer;
       size_t length;
@@ -737,23 +677,23 @@ keybox_compress (KEYBOX_HANDLE hd)
         {
           u32 last_maint = buf32_to_u32 (buffer+20);
 
-          if ( (last_maint + 3*3600) > time (NULL) )
+          if ( (last_maint + 3*3600) > make_timestamp () )
             {
-              fclose (fp);
+              es_fclose (fp);
               _keybox_release_blob (blob);
               return 0; /* Compress run not yet needed. */
             }
         }
       _keybox_release_blob (blob);
-      fseek (fp, 0, SEEK_SET);
-      clearerr (fp);
+      es_fseek (fp, 0, SEEK_SET);
+      es_clearerr (fp);
     }
 
   /* Create the new file. */
   rc = create_tmp_file (fname, &bakfname, &tmpfname, &newfp);
   if (rc)
     {
-      fclose (fp);
+      es_fclose (fp);
       return rc;;
     }
 
@@ -762,10 +702,10 @@ keybox_compress (KEYBOX_HANDLE hd)
      automagically skip any blobs flagged as deleted.  Thus what we
      only have to do is to check all ephemeral flagged blocks whether
      their time has come and write out all other blobs. */
-  cut_time = time(NULL) - 86400;
+  cut_time = make_timestamp () - 86400;
   first_blob = 1;
   skipped_deleted = 0;
-  for (rc=0; !(read_rc = _keybox_read_blob2 (&blob, fp, &skipped_deleted));
+  for (rc=0; !(read_rc = _keybox_read_blob (&blob, fp, &skipped_deleted));
        _keybox_release_blob (blob), blob = NULL )
     {
       unsigned int blobflags;
@@ -785,7 +725,7 @@ keybox_compress (KEYBOX_HANDLE hd)
                  stamp and if needed (ie. used by gpg) set the openpgp
                  flag.  */
               _keybox_update_header_blob (blob, hd->for_openpgp);
-              rc = _keybox_write_blob (blob, newfp);
+              rc = _keybox_write_blob (blob, newfp, NULL);
               if (rc)
                 break;
               continue;
@@ -829,7 +769,7 @@ keybox_compress (KEYBOX_HANDLE hd)
             }
         }
 
-      rc = _keybox_write_blob (blob, newfp);
+      rc = _keybox_write_blob (blob, newfp, NULL);
       if (rc)
         break;
     }
@@ -842,9 +782,9 @@ keybox_compress (KEYBOX_HANDLE hd)
     rc = read_rc;
 
   /* Close both files. */
-  if (fclose(fp) && !rc)
+  if (es_fclose(fp) && !rc)
     rc = gpg_error_from_syserror ();
-  if (fclose(newfp) && !rc)
+  if (es_fclose(newfp) && !rc)
     rc = gpg_error_from_syserror ();
 
   /* Rename or remove the temporary file. */
